/////////////////////////////////////////////
//Klasse: Board
//Autor: Philipp Meiner
//Datum: 11.Dez.2012
////////////////////////////////////////////
//Beschreibung und Benutzungshinweise sind in der Header Datei.

#include "board.h"
#include <QDebug>
#include <QMouseEvent>
#include <qmath.h>

Board::Board(QGraphicsView *viewPtr)
: m_view(viewPtr), VerticalFieldNumberInverted(false), GridFrameIsVisible(false),GridLinesAreVisible(false), m_MouseOverRow(-1),m_MouseOverColumn(-1)
{
  m_view->scene()->addItem(this);
  m_view->viewport()->installEventFilter(this); //Dieses Objekt zum Eventfilter machen

  ImageReloadTimer.setSingleShot(true); //Timer lst ein Event aus und hlt dann an
  ReloadDelay = 300;

  connect(&ImageReloadTimer,SIGNAL(timeout()),this,SLOT(RescaleClean()));

  GridPen = QPen(Qt::red,4,Qt::SolidLine);

  PlayingFieldFrame = m_view->scene()->addRect(0,0,0,0,GridPen);
  PlayingFieldFrame->setVisible(GridFrameIsVisible);

  this->setZValue(-100);
  PlayingFieldFrame->setZValue(-90);
}


void Board::setImage(const QString & fileName)
{
  FileNameStr = fileName;
  BoardImage.load(FileNameStr);
  this->setPixmap(QPixmap::fromImage(BoardImage).scaled(300,300, Qt::KeepAspectRatio,Qt::SmoothTransformation));
  this->setVisible(false);
  ImageReloadTimer.start(100);
}


void Board::DefinePlayingField(QRect PlayingFieldRect, int rows, int columns)
{
  PlayingField = PlayingFieldRect;
  m_RowCount = rows;
  m_ColumnCount = columns;

  qDeleteAll(FieldSeperatorLines);
  FieldSeperatorLines.clear();
  for(int i = 0; i < (m_RowCount-1)+(m_ColumnCount-1); i++){
    FieldSeperatorLines.append( m_view->scene()->addLine(0,0,0,0,GridPen));
    FieldSeperatorLines.last()->setVisible(GridLinesAreVisible);
    FieldSeperatorLines.last()->setZValue(-95);
  }
}


void Board::centerBordImageInView()
{
  ((QGraphicsPixmapItem*)this)->setPos((m_view->width()-this->pixmap().width())/2,(m_view->height()-this->pixmap().height())/2);
}


void Board::RescaleFast()
{
  setPixmap(this->pixmap().scaled(m_view->width(),m_view->height(), Qt::KeepAspectRatio,Qt::SmoothTransformation));
  centerBordImageInView(); //Spielbett in der Fenstermitte zentrieren
  DrawGrid(); //Rahmen und Trennlinien neu skalieren und positionieren
  ImageReloadTimer.start(ReloadDelay); //Timer starten, der bei ablauf ein "RescaleClean" auslst.
}

void Board::RescaleClean()
{
  this->setPixmap(QPixmap::fromImage(BoardImage).scaled(m_view->width(),m_view->height(), Qt::KeepAspectRatio,Qt::SmoothTransformation));
  centerBordImageInView(); //Spielbett in der Fenstermitte Zentrieren
  m_view->scene()->setSceneRect(0,0,m_view->width()-5,m_view->height()-5); //ntig um Mgliches Scrollen zu verhindern
  this->setVisible(true); //Spielbrett sichtbar machen
  DrawGrid(); //Rahmen und Trennlinien neu skalieren und positionieren
  emit CleanRedrawDone(); //Event auslsen, das vom Tokenmanager erfasst wird
}


void Board::setVerticalFieldNumberInversion(bool Invert){ VerticalFieldNumberInverted = Invert;}

void Board::setGridFrameVisibility(bool isVisible) {
  GridFrameIsVisible = isVisible;
  PlayingFieldFrame->setVisible(GridFrameIsVisible);
}

void Board::setGridLinesVisibility(bool isVisible)
{
  GridLinesAreVisible = isVisible;
  for(int i = 0; i < FieldSeperatorLines.size(); i++){
    FieldSeperatorLines[i]->setVisible(GridLinesAreVisible);
  }
}


void Board::setGridPen(QPen NewGridPen)
{
  GridPen = NewGridPen;
  PlayingFieldFrame->setPen(GridPen);
  for(int i = 0; i < FieldSeperatorLines.size(); i++){
    FieldSeperatorLines[i]->setPen(GridPen);
  }
}


//Zeichnet den Rahmen und die Trennlinien zwischen den Feldern. Beides wird nur angezeigt, wess es auf "visible" gesetzt wurde
void Board::DrawGrid()
{
  qreal ScaleFactor = (qreal)(this->pixmap().width())/(qreal)(BoardImage.width());
  PlayingFieldFrame->setRect(QRectF(QPointF(this->pos()+ PlayingField.topLeft() *ScaleFactor),PlayingField.size()*ScaleFactor));

  for(int i = 0; i < m_RowCount-1; i++){
    qreal Y_Val = PlayingFieldFrame->rect().top()+(i+1)*FieldHeight();
    FieldSeperatorLines[i]->setLine(PlayingFieldFrame->rect().left(),Y_Val,PlayingFieldFrame->rect().right(),Y_Val);
  }
  for(int i = 0; i < (m_ColumnCount-1); i++){
    qreal X_Val = PlayingFieldFrame->rect().left()+(i+1)*FieldWidth();
    FieldSeperatorLines[(m_RowCount-1)+i]->setLine(X_Val,PlayingFieldFrame->rect().top(),X_Val,PlayingFieldFrame->rect().bottom());
  }
}


//Gibt ein Rechteck zurck, dass die Flche des entsprechenden Feldes beschreibt
QRect Board::getFieldRect(int Row, int Column)
{
  if((Row < 0)||(Row > m_RowCount-1)||(Column < 0)||(Column > m_ColumnCount-1)){
    qDebug()<<"ERROR - getFieldRect"<<Row<<m_RowCount<<Column<<m_ColumnCount;
    return QRect(0,0,0,0);
  }
  qreal RectX = PlayingFieldFrame->rect().left() + FieldWidth() * Column;
  qreal RectY;
  if(VerticalFieldNumberInverted){
    RectY = PlayingFieldFrame->rect().top() + FieldHeight() * (m_RowCount-1-Row);
  }else{
    RectY = PlayingFieldFrame->rect().top() + FieldHeight() * Row;
  }
  return QRect(RectX,RectY,FieldWidth(),FieldHeight());
}


//Bestimmt die Koordinaten des Feldes an der bergebenen Position. Gibt "true" zurck wennn diese sich seit der letzten Messung gendert haben
//Die Membervariablen  "m_MouseOverRow" und   "m_MouseOverColumn" werden auf die ermittelten Werte gestezt
bool Board::DetermineFieldAtPos(QPointF Pos)
{
  int FieldRow, FieldColumn;
  bool hasChanged = false;

  //Feld bestimmen: Abstand des bergebenen Punktes von der oberen linken Ecke des Spielfeldbereichs durch die Breite/Hhe eines Feldes teilen.
  FieldColumn = qFloor((Pos.x()-PlayingFieldFrame->rect().left())/FieldWidth());
  FieldRow = qFloor((Pos.y()-PlayingFieldFrame->rect().top())/FieldHeight());

  if((FieldRow < 0)||(FieldRow >= m_RowCount)||(FieldColumn < 0)||(FieldColumn >= m_ColumnCount)){
    FieldRow = -1;
    FieldColumn = -1;
  }else{
    if(VerticalFieldNumberInverted){
      FieldRow = m_RowCount-1-FieldRow;
    }
  }

  if((FieldRow != m_MouseOverRow)||(FieldColumn != m_MouseOverColumn)){
    hasChanged = true;
  }
  m_MouseOverRow = FieldRow;
  m_MouseOverColumn = FieldColumn;
  return hasChanged;
}


void Board::mousePressEvent ( QGraphicsSceneMouseEvent * e )
{
  if(this->boundingRect().contains(e->pos())==false){ //Fand der Klick im Spielfeld statt?
      return;
  }
  DetermineFieldAtPos(e->pos()+this->pos());
  emit MouseClickedOnBoard(m_MouseOverRow, m_MouseOverColumn, e->button());
}


bool Board::eventFilter(QObject* object, QEvent* event)
{
    Q_UNUSED(object);
  if(event->type()==QEvent::MouseMove){
    if(DetermineFieldAtPos(static_cast<QMouseEvent*>(event)->localPos())){//Neues Feld ist unter der Maus
      emit MouseOverFieldChanged(m_MouseOverRow, m_MouseOverColumn);
    }
    return true;
  }
  return false; //Keine Mausbewegung  Event an den View weiterleiten
}
