/////////////////////////////////////////////
//Klasse: Board
//Autor: Philipp Meiner
//Datum: 11.Dez.2012
////////////////////////////////////////////
//Beschreibung:
//Diese Klasse ist in der Lage eine ihr bergebene Spielfeldgrafik skaliert
//anzuzeigen und die Positionen der Felder zu verwalten.
//
//Benutzung:
//Die Spielfeldgrafik wird in dem View, der dem Konstruktor bergeben wird angezeigt.
//Dabei wird sie auf die maximale Gre, bei der sie noch nicht verzerrt wird, skaliert.
//
//Nach der Erstellung muss die Funktion DefinePlayingField aufgerufen werden.
//Das ihr bergebene QRect gibt die Position der Spielfelder in der Spielfeldgrafik in Pixeln an.
//Beispiel: Ein Rechteck mit der Position (5,8), der Breite 60 und der Hhe 70 sagt aus,
//dass die linke obere Ecke des Spielfeldbereis an der Position des Pixels mit den Koordinaten (5,8) liegt
//und der Bereich 60 Pixel breit und 70 Pixel hoch ist.
//Zur berprfung kann man ber die Funktionen setGridFrameVisibility, und setGridLinesVisibility Linien anzeigen
//lassen, die die Positionen der Spielfelder zeigen. Mit setGridPen kann man das Aussehen dieser Linen ndern.
//
//Die Funktion getFieldRect gibt ein Rechteck zurck, dass die Gre und Position der Flche des angegebenen Feldes
//enthlt. Die Zhlweise der Reihen und Spalten startet bei 0. Mit der Funktion setVerticalFieldNumberInversion kann
//bestimmt werden, ob die 0. Reihe ganz oben oder ganz unten ist.
//
//Jedes Mal wenn sich die Gre des Views ndert muss die Funktion RescaleFast aufgerufen werden, damit das
//Spielfeld auf die neue Gre angepasst werden kann. Ein sauberes Neuzeichnen mit der neuen Grer passiert jedoch
//erst nach eine kleinen Verzgerung, die mit der Funktion setReloadDelayInMS eingestellt werden kann.



#ifndef BOARD_H
#define BOARD_H

#include <QObject>
#include <QGraphicsView>
#include <QGraphicsPixmapItem>
#include <QGraphicsLineItem>
#include <QGraphicsRectItem>
#include <QGraphicsSceneMouseEvent>

#include <QPen>
#include <QRect>
#include <QString>
#include <QImage>
#include <QTimer>
#include <QList>


class Board : public QObject, private QGraphicsPixmapItem
{
  Q_OBJECT
public:
  Board(QGraphicsView* viewPtr);
  void setImage(const QString & fileName);

  void RescaleFast();
  void setReloadDelayInMS(int newReloadDelay);

  void DefinePlayingField(QRect PlayingFieldRect, int rows, int columns);

  void setVerticalFieldNumberInversion(bool Invert);
  void setGridFrameVisibility(bool isVisible);
  void setGridLinesVisibility(bool isVisible);

  void setGridPen(QPen NewGridPen);

  QRect getFieldRect(int Row, int Column);

  int RowCount() const;
  int ColumnCount() const;
  qreal FieldWidth() const;
  qreal FieldHeight() const;

  QGraphicsView* viewPtr() const;

signals:
  void CleanRedrawDone();
  void MouseOverFieldChanged(int Row, int Column);
  void MouseClickedOnBoard(int Row, int Column, Qt::MouseButton Button);

private slots:
  void RescaleClean();

private:

  QString FileNameStr;
  QGraphicsView* m_view;

  QTimer ImageReloadTimer;
  int ReloadDelay;

  QImage BoardImage;

  QRect PlayingField;
  QGraphicsRectItem * PlayingFieldFrame;
  QList<QGraphicsLineItem*> FieldSeperatorLines;
  int m_RowCount, m_ColumnCount;
  bool VerticalFieldNumberInverted;

  int MouseOverRow() const;
  int MouseOverColumn() const;

  QPen GridPen;
  bool GridFrameIsVisible;
  bool GridLinesAreVisible;

  void centerBordImageInView();
  void DrawGrid();
  bool DetermineFieldAtPos(QPointF Pos);

  int m_MouseOverRow;
  int m_MouseOverColumn;

protected:
  virtual void mousePressEvent ( QGraphicsSceneMouseEvent * e );
  bool eventFilter(QObject* object, QEvent* event);
};

inline int Board::RowCount() const{ return m_RowCount;}
inline int Board::ColumnCount() const{ return m_ColumnCount;}

inline qreal Board::FieldWidth() const{ return PlayingFieldFrame->rect().width()/m_ColumnCount;}
inline qreal Board::FieldHeight() const{ return PlayingFieldFrame->rect().height()/m_RowCount;}

inline int Board::MouseOverRow() const {return m_MouseOverRow;}
inline int Board::MouseOverColumn() const {return m_MouseOverColumn;}

inline QGraphicsView* Board::viewPtr() const {return m_view;}

inline void Board::setReloadDelayInMS(int newReloadDelay){ ReloadDelay = newReloadDelay;}

#endif // BOARD_H
