#include "gameuisge.h"
#include <QDir>
#include <QDebug>
#include "messagedialog.h"
#include <QApplication>

GameUisge::GameUisge(Board *BoardPtr, TokenManager *TokenManagerPtr, QObject *parent)
:  QObject(parent), TheBoard(BoardPtr), TheTokenManager(TokenManagerPtr), m_RowCount(6), m_ColumnCount(7)
{
  ConfigBoard();
  InitTokenManager();
  InitRuleChecker();
  connect(TheTokenManager,SIGNAL(MovingDone()),this,SLOT(MovingDone()));
  SetUpNewGame();
}


void GameUisge::ConfigBoard()
{
  TheBoard->setImage(QDir::currentPath()+QDir::separator()+"Board.png");
  TheBoard->DefinePlayingField(QRect(140,140,831,713),m_RowCount, m_ColumnCount);
  TheBoard->setGridPen(QPen(Qt::gray,4,Qt::SolidLine) );
  TheBoard->setGridFrameVisibility(false);
  TheBoard->setGridLinesVisibility(false);
  TheBoard->setVerticalFieldNumberInversion(false);

  connect(TheBoard,SIGNAL(MouseClickedOnBoard(int,int,Qt::MouseButton)),this,SLOT(MouseClickedOnBoard(int,int,Qt::MouseButton)));
  connect(TheBoard,SIGNAL(MouseOverFieldChanged(int,int)),this,SLOT(MouseOverFieldChanged(int,int)));
}


void GameUisge::InitTokenManager()
{
  TheTokenManager->setMovingSpeed(5);

  TypeIdWhitePawn = TheTokenManager->AddTokenType(QDir::currentPath()+QDir::separator()+"PawnWhite.png",PlayerWhite,100);
  TypeIdWhiteCrown = TheTokenManager->AddTokenType(QDir::currentPath()+QDir::separator()+"KingWhite.png",PlayerWhite,100);
  TypeIdBlackPawn = TheTokenManager->AddTokenType(QDir::currentPath()+QDir::separator()+"PawnBlack.png",PlayerBlack,100);
  TypeIdBlackCrown = TheTokenManager->AddTokenType(QDir::currentPath()+QDir::separator()+"KingBlack.png",PlayerBlack,100);

  TypeIdMouseOverMarker = TheTokenManager->AddTokenType(QDir::currentPath()+QDir::separator()+"MarkerMouseOver.png",PlayerNone,110,1.1);
  TypeIdSelectedMarker = TheTokenManager->AddTokenType(QDir::currentPath()+QDir::separator()+"MarkerYellow.png",PlayerNone,20);

  TypeIdPossibleMoveMarker = TheTokenManager->AddTokenType(QDir::currentPath()+QDir::separator()+"MarkerLightGreen.png",PlayerNone,20);
  TypeIdLastMoveFromLMarker = TheTokenManager->AddTokenType(QDir::currentPath()+QDir::separator()+"MarkerLastMoveFrom.png",PlayerNone,10);
  TypeIdLastMoveDestLMarker = TheTokenManager->AddTokenType(QDir::currentPath()+QDir::separator()+"MarkerLastMoveTo.png",PlayerNone,10);

  MouseOverMarker = TheTokenManager->AddToken(TypeIdMouseOverMarker);
  SelectedMarker = TheTokenManager->AddToken(TypeIdSelectedMarker);
  LastMoveFromMarker = TheTokenManager->AddToken(TypeIdLastMoveFromLMarker);
  LastMoveDestMarker = TheTokenManager->AddToken(TypeIdLastMoveDestLMarker);
}


void GameUisge::InitRuleChecker()
{
  TheRuleChecker = new RuleChecker(TheTokenManager);

  TheRuleChecker->AddTokenID(TypeIdWhitePawn, false);
  TheRuleChecker->AddTokenID(TypeIdWhiteCrown, true);
  TheRuleChecker->AddTokenID(TypeIdBlackPawn, false);
  TheRuleChecker->AddTokenID(TypeIdBlackCrown, true);
}


void GameUisge::DeleteAllTokensOfType(int Type)
{
   while(TheTokenManager->TokensCountForType(Type)){
      TheTokenManager->DeleteToken(TheTokenManager->getTokenWithType(Type));
   }
}


void GameUisge::SetUpNewGame()
{
  DeleteAllTokensOfType(TypeIdWhitePawn);
  DeleteAllTokensOfType(TypeIdWhiteCrown);
  DeleteAllTokensOfType(TypeIdBlackPawn);
  DeleteAllTokensOfType(TypeIdBlackCrown);
  DeleteAllTokensOfType(TypeIdPossibleMoveMarker);

  TheTokenManager->AddToken(TypeIdWhitePawn,3,2);
  TheTokenManager->AddToken(TypeIdWhitePawn,3,3);
  TheTokenManager->AddToken(TypeIdWhitePawn,3,4);
  TheTokenManager->AddToken(TypeIdWhitePawn,3,5);
  TheTokenManager->AddToken(TypeIdWhitePawn,4,3);
  TheTokenManager->AddToken(TypeIdWhitePawn,4,4);

  TheTokenManager->AddToken(TypeIdBlackPawn,1,2);
  TheTokenManager->AddToken(TypeIdBlackPawn,1,3);
  TheTokenManager->AddToken(TypeIdBlackPawn,2,1);
  TheTokenManager->AddToken(TypeIdBlackPawn,2,2);
  TheTokenManager->AddToken(TypeIdBlackPawn,2,3);
  TheTokenManager->AddToken(TypeIdBlackPawn,2,4);

  SelectedMarker->setVisibility(false);
  MouseOverMarker->setVisibility(false);

  m_MoveCounter = 0;
  m_GameFinished = false;
  MoveHistory.clear();
  LastMoveFromMarker->setVisibility(false);
  LastMoveDestMarker->setVisibility(false);

  ActivePlayer = PlayerWhite;
}


//Pruft die Zugmglichkeiten des Steins an der bergebenen Position. Ist PlaceMarker == true werden die Zugmglichkeiten durch Markierungen angezeigt.
//Rckgabewert = Anzahl der mglichen Zge.
int GameUisge::CheckMovesAndPlaceMarker(int Row, int Column, bool PlaceMarker)
{
  int PossibleMoves = 0;
  int OffestX[] = { 1, 1, 1, 0, 0,-1,-1,-1, 2,-2, 0, 0};
  int OffestY[] = { 1, 0,-1, 1,-1, 1, 0,-1, 0, 0, 2,-2};
  for(int i = 0; i < 12; i++){
    if(TheRuleChecker->CheckMove(Row,Column,Row+OffestX[i] , Column+OffestY[i]) == 0){
      if(PlaceMarker){
         TheTokenManager->AddToken(TypeIdPossibleMoveMarker, Row+OffestX[i], Column+OffestY[i])->setVisibility(m_ShowPossibleMoves);
      }
      PossibleMoves++;
    }
  }
  return PossibleMoves;
}


void GameUisge::SelectTokenAndCheckPossibleMoves(int Row, int Column)
{
  DeselectToken();//Zum Aufrumen ntig
  TheTokenManager->PlaceTokenOnField(TheTokenManager->getNrOfToken(SelectedMarker),Row,Column);
  SelectedMarker->setVisibility(true);
  CheckMovesAndPlaceMarker(Row, Column, true);
}


void GameUisge::DeselectToken()
{
  SelectedMarker->setVisibility(false);
  DeleteAllTokensOfType(TypeIdPossibleMoveMarker);
}


void GameUisge::FlipToken(Token* TokenToFlip)
{
  if(TokenToFlip == 0){qDebug()<<"ERROR - nullpointer"; return;}

  int TokenNr = TheTokenManager->getNrOfToken(TokenToFlip);
  if(TokenToFlip->Type() == TypeIdWhitePawn){TheTokenManager->setTokenType(TokenNr,TypeIdWhiteCrown); return;}
  if(TokenToFlip->Type() == TypeIdWhiteCrown){TheTokenManager->setTokenType(TokenNr,TypeIdWhitePawn); return;}
  if(TokenToFlip->Type() == TypeIdBlackPawn){TheTokenManager->setTokenType(TokenNr,TypeIdBlackCrown); return;}
  if(TokenToFlip->Type() == TypeIdBlackCrown){TheTokenManager->setTokenType(TokenNr,TypeIdBlackPawn); return;}
  qDebug()<<"ERROR - UnkownTokenType";
}


void GameUisge::ChangeActivePlayer()
{
  if(ActivePlayer == PlayerWhite){ActivePlayer = PlayerBlack; return;}
  if(ActivePlayer == PlayerBlack){ActivePlayer = PlayerWhite; return;}
  qDebug()<<"ERROR - UnkownPlayer";
}


//Prft ob der nchste Spieler ziehen kann und ob der Jetzige Spieler 6 Kronesteine hat
bool GameUisge::PlayerHasWon(Player PlayerToTest)
{
    Q_UNUSED(PlayerToTest);
  int CrownTypeID = 0;
  Player NextPlayer;
  if(ActivePlayer == PlayerWhite){NextPlayer = PlayerBlack; CrownTypeID = TypeIdWhiteCrown;}
  if(ActivePlayer == PlayerBlack){NextPlayer = PlayerWhite; CrownTypeID = TypeIdBlackCrown;}

  //Zugmglichkeiten des nchsten Spielers zhlen
  Token* TempToken;
  int PossibleMoves = 0;
  for(int i = 0; i < TheTokenManager->TokensCountForPlayer(NextPlayer);i++){
    TempToken = TheTokenManager->getTokenOfPlayer(NextPlayer,i);
    PossibleMoves += CheckMovesAndPlaceMarker(TempToken->Row(),TempToken->Column(),false);
  }
  if(PossibleMoves == 0){return true;}

  return (TheTokenManager->TokensCountForType(CrownTypeID) >= 6);
}



void GameUisge::DisplayWinMessage()
{
  MessageDialog M;
  if(ActivePlayer == PlayerWhite){M.setHTMLText("<div align=\"center\"><h1><b><br>Wei hat gewonnen</b></h1></div>");}
  if(ActivePlayer == PlayerBlack){M.setHTMLText("<div align=\"center\"><h1><b><br>Schwarz hat gewonnen</b></h1></div>");}
  M.setWindowTitle("Spiel zu Ende");
  M.move(QApplication::activeWindow()->frameGeometry().center()-QPoint(M.frameGeometry().width()/2,M.frameGeometry().height()/2));
  M.exec();
}


//Zieht den Stein an der Position des Selection-Markers auf die bergebene Position. Der Zug wird jedoch nur ausgefhrt, wenn er regelkonform ist.
void GameUisge::TryMoveToField(int Row, int Column)
{
  Token* TokenToMove = TheRuleChecker->GetTokenOnField(SelectedMarker->Row(),SelectedMarker->Column());
  if(TokenToMove){
    int MoveCheckResult = TheRuleChecker->CheckMove(TokenToMove->Row(),TokenToMove->Column(),Row,Column);

    if(MoveCheckResult == 0){//Situation nach Zug erlaubt
        int TokenNr = TheTokenManager->getNrOfToken(TokenToMove);
        TheTokenManager->setTokenMoveDestination(TokenNr,Row,Column);
        if((abs(TokenToMove->Row()-Row)==2)||(abs(TokenToMove->Column()-Column)==2)){
          FlipToken(TokenToMove);
        }
        TheTokenManager->MoveTokens();
        MoveData NewMove = {TokenToMove->Row(),TokenToMove->Column(),Row,Column};
        MoveHistory.append(NewMove);
        m_MoveCounter++;        

        TheTokenManager->PlaceTokenOnField(TheTokenManager->getNrOfToken(LastMoveFromMarker),TokenToMove->Row(),TokenToMove->Column());
        LastMoveFromMarker->setVisibility(m_ShowLastMove);

        TheTokenManager->PlaceTokenOnField(TheTokenManager->getNrOfToken(LastMoveDestMarker),Row,Column);
        LastMoveDestMarker->setVisibility(m_ShowLastMove);

    }else{
      //Zug nicht mglich - Erklrung
    }
  }else{
    qDebug()<<"ERROR - Selector at field withot token!";
  }
}


//Wird nach dem ablauf der Bewegungsanimation eines Zuges aufgerufen.
void GameUisge::MovingDone()
{
  if(PlayerHasWon(ActivePlayer)){
    m_GameFinished = true;
    DisplayWinMessage();
  }
  ChangeActivePlayer();
}


void GameUisge::UnDoLastMove()
{
  if(MoveHistory.isEmpty()){return;}
  DeselectToken();
  MoveData MoveToUndo = MoveHistory.last();
  Token* TokenToMove = TheRuleChecker->GetTokenOnField(MoveToUndo.RowTo,MoveToUndo.ColumnTo);
  if(TokenToMove == 0){qDebug()<<"ERROR"; return;}
  if((abs(MoveToUndo.RowFrom-MoveToUndo.RowTo)==2)||(abs(MoveToUndo.ColumnFrom-MoveToUndo.ColumnTo)==2)){
    FlipToken(TokenToMove);
  }
  TheTokenManager->PlaceTokenOnField(TheTokenManager->getNrOfToken(TokenToMove),MoveToUndo.RowFrom,MoveToUndo.ColumnFrom);
  m_MoveCounter--;
  m_GameFinished = false;
  MoveHistory.removeLast();
  if(!MoveHistory.isEmpty()){
      TheTokenManager->PlaceTokenOnField(TheTokenManager->getNrOfToken(LastMoveFromMarker),MoveHistory.last().RowFrom,MoveHistory.last().ColumnFrom);
      TheTokenManager->PlaceTokenOnField(TheTokenManager->getNrOfToken(LastMoveDestMarker),MoveHistory.last().RowTo,MoveHistory.last().ColumnTo);
  }else{
      LastMoveFromMarker->setVisibility(false);
      LastMoveDestMarker->setVisibility(false);
  }
  ChangeActivePlayer();
}


//Verarbeitung eines Mausklicks
void GameUisge::MouseClickedOnBoard(int Row, int Column, Qt::MouseButton Button)
{
  if(TheTokenManager->MovingInProgress() || m_GameFinished){return;}

  if(Button == Qt::LeftButton){
    Token* TokenOnField = TheRuleChecker->GetTokenOnField(Row,Column);
    if(TokenOnField){
      if(TokenOnField->Player()== ActivePlayer){
        if((SelectedMarker->Row() != Row)||(SelectedMarker->Column() != Column)||(SelectedMarker->Visibility() == false)){
          //Angeklickter Spieltein bisher nicht ausgewhlt;
          SelectTokenAndCheckPossibleMoves(Row, Column);
        }else{
          DeselectToken();
        }
      }else{      
        //Falscher Player
        DeselectToken();
      }
    }else{
      if(SelectedMarker->Visibility()==true){//Es ist ein Stein Ausgewhlt
        TryMoveToField(Row, Column);
      }
      DeselectToken();
    }
  }
}


void GameUisge::MouseOverFieldChanged(int Row, int Column){
  if(Row >= 0){
    TheTokenManager->PlaceTokenOnField(TheTokenManager->getNrOfToken(MouseOverMarker),Row,Column);
    MouseOverMarker->setVisibility(m_ShowMouseOver);
  }else{
    MouseOverMarker->setVisibility(false);
  }
}


void GameUisge::setShowLastMove(bool Show)
{
  m_ShowLastMove = Show;
  if(m_MoveCounter != 0){
    LastMoveFromMarker->setVisibility(m_ShowLastMove);
    LastMoveDestMarker->setVisibility(m_ShowLastMove);
  }
}

void GameUisge::setShowPossibleMoves(bool Show)
{
  m_ShowPossibleMoves = Show;
  for(int i = 0; i < TheTokenManager->TokensCountForType(TypeIdPossibleMoveMarker); i++){
    TheTokenManager->getTokenWithType(TypeIdPossibleMoveMarker,i)->setVisibility(m_ShowPossibleMoves);
  }
}

void GameUisge::setShowMouseOver(bool Show)
{
  m_ShowMouseOver = Show;
}
