/////////////////////////////////////////////
//Klasse: Board
//Autor: Philipp Meiner
//Datum: 11.Dez.2012
////////////////////////////////////////////
//Beschreibung:
//Diese Klasse implementiert den Spielablauf des Spiels Uisge.
//


#ifndef GAMEUISGE_H
#define GAMEUISGE_H

#include <QObject>

#include <QList>

#include "tokenmanager.h"
#include "board.h"
#include "token.h"
#include "rulechecker.h"


enum Player{PlayerNone = 0, PlayerWhite, PlayerBlack};

struct MoveData
{
  int RowFrom;
  int ColumnFrom;
  int RowTo;
  int ColumnTo;
};


class GameUisge : public QObject
{
  Q_OBJECT
public:
  explicit GameUisge(Board * BoardPtr, TokenManager* TokenManagerPtr, QObject *parent = 0);
  
signals:
  
public slots:
  void SetUpNewGame();
  void MouseOverFieldChanged(int Row, int Column);
  void MouseClickedOnBoard(int Row, int Column, Qt::MouseButton Button);
  void setShowLastMove(bool Show);
  void setShowPossibleMoves(bool Show);
  void setShowMouseOver(bool Show);
  void UnDoLastMove();
  void MovingDone();

private:
  void DeleteAllTokensOfType(int Type);

  void ConfigBoard();
  void InitTokenManager();
  void InitRuleChecker();

  void DeselectToken();
  void SelectTokenAndCheckPossibleMoves(int Row, int Column);
  int CheckMovesAndPlaceMarker(int Row, int Column, bool PlaceMarker);

  void TryMoveToField(int Row, int Column);
  void FlipToken(Token* TokenToFlip);
  void ChangeActivePlayer();
  bool PlayerHasWon(Player PlayerToTest);

  void DisplayWinMessage();

  QList<MoveData>MoveHistory;

  Board * TheBoard;

  RuleChecker * TheRuleChecker;
  TokenManager * TheTokenManager;
  Player ActivePlayer;

  Token* MouseOverMarker;
  Token* SelectedMarker;
  Token* LastMoveFromMarker;
  Token* LastMoveDestMarker;

  bool m_ShowLastMove;
  bool m_ShowPossibleMoves;
  bool m_ShowMouseOver;
  bool m_GameFinished;
  int m_MoveCounter;

  const int m_RowCount;
  const int m_ColumnCount;

  int TypeIdWhitePawn;
  int TypeIdWhiteCrown;
  int TypeIdBlackPawn;
  int TypeIdBlackCrown;
  int TypeIdMouseOverMarker;
  int TypeIdSelectedMarker;
  int TypeIdPossibleMoveMarker;
  int TypeIdLastMoveFromLMarker;
  int TypeIdLastMoveDestLMarker;
};

#endif // GAMEUISGE_H
