﻿#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "messagedialog.h"

#include <QGraphicsScene>

#include <QDir>
#include <QDebug>
#include <qmath.h>


MainWindow::MainWindow(QWidget *parent) :
  QMainWindow(parent),
  ui(new Ui::MainWindow)
{
  ui->setupUi(this);

  QGraphicsScene *MainScene = new QGraphicsScene(this); //Erstellen einer neuen Scene
  ui->MainView->setScene(MainScene); //Zuweisen der Scene zum im Formulareditor erstellen View
  ui->MainView->setAlignment(Qt::AlignLeft | Qt::AlignTop); //Ausrichten: Oben Links = Pos 0/0

  QRadialGradient gradient(0, 0, 628);
  gradient.setSpread(QGradient::ReflectSpread);
  gradient.setColorAt(0,Qt::darkGray);
  gradient.setColorAt(1,Qt::black);
  ui->MainView->scene()->setBackgroundBrush(gradient);

  TheBoard = new Board(ui->MainView);
  TheTokenManager = new TokenManager(TheBoard);

  TheGame = new GameUisge(TheBoard,TheTokenManager);

  connect(ui->actionNeu,SIGNAL(triggered()),TheGame,SLOT(SetUpNewGame()));

  connect(ui->actionShowPossibleMoves,SIGNAL(toggled(bool)),TheGame,SLOT(setShowPossibleMoves(bool)));
  TheGame->setShowPossibleMoves(ui->actionShowPossibleMoves->isChecked());

  connect(ui->actionShowLastMove,SIGNAL(toggled(bool)),TheGame,SLOT(setShowLastMove(bool)));
  TheGame->setShowLastMove(ui->actionShowLastMove->isChecked());

  connect(ui->actionMouseOver,SIGNAL(toggled(bool)),TheGame,SLOT(setShowMouseOver(bool)));
  TheGame->setShowMouseOver(ui->actionMouseOver->isChecked());

  connect(ui->actionUnDo,SIGNAL(triggered()),TheGame,SLOT(UnDoLastMove()));

  connect(ui->actionInfo,SIGNAL(triggered()),this,SLOT(OpenInfo()));
  connect(ui->actionManual,SIGNAL(triggered()),this,SLOT(OpenManual()));

  connect(ui->actionExit,SIGNAL(triggered()),qApp,SLOT(quit()));
}

MainWindow::~MainWindow()
{
  delete ui;
}


void MainWindow::resizeEvent(QResizeEvent * event)
{ 
    Q_UNUSED(event);
  TheBoard->RescaleFast();
  TheTokenManager->ReplaceAllTokensFast();
  if(ui->MainView->width() > 100){
    QRadialGradient gradient(0, 0, 0.5*qSqrt((ui->MainView->width()*ui->MainView->width())+(ui->MainView->height()*ui->MainView->height())));
    gradient.setSpread(QGradient::ReflectSpread);
    gradient.setColorAt(0,Qt::darkGray);
    gradient.setColorAt(1,Qt::black);
    ui->MainView->scene()->setBackgroundBrush(gradient);
  }
}

void MainWindow::OpenInfo()
{
  MessageDialog M;
  M.setHTMLText("<b>UISGE</b> V1.0<br>"
            "Ein Programm von Philipp Meißner<br><br>"
            "entstanden im Rahmen einer Projektarbeit<br>"
            "an der Fachhochschule Pforzheim<br><br>"
            "erstellt mit Qt "QT_VERSION_STR);
  M.setWindowTitle("Info");
  M.resize(M.width(),150);
  M.move(this->frameGeometry().center()-QPoint(M.frameGeometry().width()/2,M.frameGeometry().height()/2));
  M.exec();
}

void MainWindow::OpenManual()
{
  MessageDialog M;
  M.setHTMLText(
  "<b>Ziel des Spiels</b><br>"
  "Ziel des Spiels ist es, als erster alle eigenen Steine mit der Krone nach oben zu legen.<br><br>"

  "<b>Spielverlauf</b><br>"
  "Abwechselnd springt jeweils ein Stein über einen anderen, eigenen oder gegnerischen, Stein in waagerechter oder senkrechter Richtung.<br>"
  "Das Landefeld muss frei sein. Der springende Stein wird herumgedreht. So wird aus einem leeren Stein ein Kronestein und umgekehrt.<br>"
  "Kronesteine dürfen, im gegensatz zu den leeren Steinen, nicht nur springen sondern auch ein Feld in waagerechte senkrechter oder diagonaler Richtung ziehen. Beim Ziehen werden Kronesteine nicht herumgedreht.<br><br>"

  "Bedingung: Jedes Ziehen oder Springen ist nur dann erlaubt, wenn anschließend alle 12 Steine in waagerechter oder senkrechter Richtung miteinander verbunden bleiben.<br><br>"

  "<b>Ende des Spiels</b><br>"
  "Sobald es einem Spieler gelungen ist, alle seine Steine mit der Konenseite nach oben zu drehen, hat er gewonnen.<br><br>"

  "Kommt ein Spieler in eine Situation, in der er keinen gültigen Zug ausführen kann, hat er verloren."
  );
  M.setWindowTitle("Spielanleitung");
  M.resize(M.width(),450);
  M.move(this->frameGeometry().center()-QPoint(M.frameGeometry().width()/2,M.frameGeometry().height()/2));
  M.exec();
}
