/////////////////////////////////////////////
//Klasse: RuleChecker
//Autor: Philipp Meiner
//Datum: 12.Dez.2012
/////////////////////////////////////////////
//Beschreibung und Benutzungshinweise sind in der Header Datei.

#include "rulechecker.h"
#include <QDebug>

RuleChecker::RuleChecker(TokenManager* TokenManagerPtr): TheTokenManager(TokenManagerPtr)
{
}


Token* RuleChecker::GetTokenOnField(int Row, int Column)
{
  Token* TempToken;
  int TempType;
  int TokensCount = TheTokenManager->TokensCountOnField(Row, Column);
  for(int i = 0; i < TokensCount; i++){
    TempToken = TheTokenManager->getTokenOnField(Row, Column, i);
    if(TempToken){
      TempType = TempToken->Type();
      for(int k = 0; k < ListOfTokenIDs.size();k++){
          if(TempType == ListOfTokenIDs[k]){
            return TempToken;
          }
      }
    }
  }
  return 0;
}


void RuleChecker::RecursiveFloodFill4Check(int Row, int Column)
{
  //Prfen ob Feld in der Liste enthalten und noch nicht besucht
  for(int i = 0; i<ListOfOccupiedFields.size(); i++){
    if((ListOfOccupiedFields[i].Row == Row)&&(ListOfOccupiedFields[i].Column == Column)&&(ListOfOccupiedFields[i].Visited == false)){

      ListOfOccupiedFields[i].Visited = true;
      FieldsVisited++;

      //Algorithmus fr alle horizontal und vertikal angrenzenden Felder ausfhren
      RecursiveFloodFill4Check(Row+1, Column);
      RecursiveFloodFill4Check(Row-1, Column);
      RecursiveFloodFill4Check(Row, Column+1);
      RecursiveFloodFill4Check(Row, Column-1);
    }
  }
  return;
}


bool RuleChecker::CheckConnection()
{
  //alle Felder als noch nicht besucht markieren
  FieldsVisited = 0;
  for(int i = 0; i<ListOfOccupiedFields.size(); i++){
    ListOfOccupiedFields[i].Visited = false;
  }
  //Floodfillalgorithmus starten
  RecursiveFloodFill4Check(ListOfOccupiedFields[0].Row,ListOfOccupiedFields[0].Column);
  return (FieldsVisited == ListOfOccupiedFields.size());//Wurden alle Felder besucht?
}


int RuleChecker::DoFullCheck()
{
  if(CheckForDoublets()){ return 1; } //Feld Doppelt Besetzt
  if(CheckBoarders()){ return 2; } //Stein auerhalb der Brettgrenzen
  if(!CheckConnection()){return 3;} //Steine nicht alle Horizontal/Vertikal verbunden
  return 0;
}


int RuleChecker::CheckMove(int RowFrom, int ColumnFrom, int RowTo, int ColumnTo)
{
  if((RowFrom == RowTo)&&(ColumnFrom == ColumnTo)){return 6;}

  Token* TokenToMove = GetTokenOnField(RowFrom,ColumnFrom);
  if(TokenToMove == 0){return 4;} //Kein Stein au dem "From-Feld"

  bool TokenCanMoveThisWay = false;
  if((abs(RowFrom-RowTo) <= 1) && (abs(ColumnFrom-ColumnTo) <= 1)){ //Bewegungszug
    for(int i = 0; i<ListOfCrownTokenIDs.size();i++){
      if(TokenToMove->Type() == ListOfCrownTokenIDs[i]){//Es handelt sich um einen Kronestein
        TokenCanMoveThisWay = true;
      }
    }
  }else if(((abs(RowFrom-RowTo)==2)&&(abs(ColumnFrom-ColumnTo)==0)) || ((abs(RowFrom-RowTo)==0)&&(abs(ColumnFrom-ColumnTo)==2))){//Sprungzug
    if(GetTokenOnField((int)((RowFrom+RowTo)/2), (int) ((ColumnFrom+ColumnTo)/2)) != 0){//Es liegt ein Stein zum Drberspringen da
      TokenCanMoveThisWay = true;
    }
  }
  if(TokenCanMoveThisWay == false){return 5;}
  LoadListOfOccupiedFields();
  if(ListOfOccupiedFieldsDoVirtualMove(RowFrom, ColumnFrom, RowTo, ColumnTo)){return 4;}//Kein Stein au dem "From-Feld" sollte hier nicht mehr auftreten;
  return DoFullCheck();
}


//Prft, dass es keine 2 Listeneintrge mit den selben Koordinaten gibt
int RuleChecker::CheckForDoublets()
{
  int DoubletCount = 0;
  int ListSize = ListOfOccupiedFields.size();
  for(int i = 0; i< ListSize; i++){
    for(int k = 0; k< ListSize; k++){
      if(i != k){
        if((ListOfOccupiedFields[i].Row == ListOfOccupiedFields[k].Row)&&(ListOfOccupiedFields[i].Column == ListOfOccupiedFields[k].Column)){
          DoubletCount++;
        }
      }
    }
  }
  return DoubletCount;
}


//Pruft, dass kein Listeneintrag Koordinaten hat, die auerhalb des Spielfelds liegen
bool RuleChecker::CheckBoarders()
{
  for(int i = 0; i<ListOfOccupiedFields.size();i++){
    if((ListOfOccupiedFields[i].Row < 0)||(ListOfOccupiedFields[i].Row >= TheTokenManager->RowCount())||(ListOfOccupiedFields[i].Column < 0)||(ListOfOccupiedFields[i].Column >= TheTokenManager->ColumnCount())){
      return true;
    }
  }
  return false;
}


//Der Listeneintrags mit den Koordinaten des Startfeldes wird gesucht. Anschlieend werden seine Koordinaten auf die des Zielfeldes gesetzt.
bool RuleChecker::ListOfOccupiedFieldsDoVirtualMove(int RowFrom, int ColumnFrom, int RowTo, int ColumnTo)
{
  for(int i = 0; i<ListOfOccupiedFields.size();i++){
    if((ListOfOccupiedFields[i].Row == RowFrom)&&(ListOfOccupiedFields[i].Column == ColumnFrom)){
      ListOfOccupiedFields[i].Row = RowTo;
      ListOfOccupiedFields[i].Column = ColumnTo;
      return false;
    }
  }
  return true;
}


//Alle Spielsteinpositionen werden in eine interne temporre Liste geladen
void RuleChecker::LoadListOfOccupiedFields()
{
  ListOfOccupiedFields.clear();
  Token* TempToken;
  int ItemsCount;
  for(int i = 0; i<ListOfTokenIDs.size();i++){
    ItemsCount = TheTokenManager->TokensCountForType(ListOfTokenIDs[i]);
    for(int k = 0; k < ItemsCount; k++){
      TempToken = TheTokenManager->getTokenWithType(ListOfTokenIDs[i],k);

      FieldWithToken NewField;
      NewField.Row = TempToken->Row();
      NewField.Column  = TempToken->Column();
      NewField.Visited = false;
      ListOfOccupiedFields.append(NewField);
    }
  }
}


void RuleChecker::AddTokenID(int ID, bool isCrown){
  ListOfTokenIDs.append(ID);
  if(isCrown){
    ListOfCrownTokenIDs.append(ID);
  }
}


void RuleChecker::RemoveAllTokenIDs(){
  ListOfTokenIDs.clear();
  ListOfCrownTokenIDs.clear();
}
