/////////////////////////////////////////////
//Klasse: RuleChecker
//Autor: Philipp Meiner
//Datum: 12.Dez.2012
/////////////////////////////////////////////
//Beschreibung:
//Diese Klasse kann Zge auf ihre Regelkonformitt prfen.
//
//Benutung:
//Der Klasse muss im Konstruktor ein Zeiger auf den Tokenmanager bergeben werden, der die Spielsteine auf dem Brett verwaltet.
//Anschlieend mssen mit der Methode AddTokenID die TokenType IDs der Spielsteine bergeben werden wobei mit dem Bool-Parameter
//isCrown festgelegt wird, ob es sich bei dem TokenType mit der bergebenen ID um einen leeren oder einen Kronestein handelt.
//
//Die Wichtigste Methode ist CheckMove. Ihr werden die Koordinaten des Start- und Zielfeldes eines Zuges bergeben und die Methode
//prft, ob der Zug Regelkonform ist.
//
//Die Methode GetTokenOnField gibt einen Zeiger auf den Spielstein an der bergebenen Position zurck. Ist das Feld leer wird 0 zurck gegeben.


#ifndef RULECHECKER_H
#define RULECHECKER_H


#include <QList>
#include "tokenmanager.h"
#include "token.h"


struct FieldWithToken
{
  int Row;
  int Column;
  bool Visited;
};

class RuleChecker
{
public:
  RuleChecker(TokenManager *TokenManagerPtr);
  void AddTokenID(int ID, bool isCrown = false);
  void RemoveAllTokenIDs();

  int CheckMove(int RowFrom, int ColumnFrom, int RowTo, int ColumnTo);

  Token* GetTokenOnField(int Row, int Column);

private:
  TokenManager * TheTokenManager;

  int FieldsVisited;

  void RecursiveFloodFill4Check(int Row, int Column);
  bool CheckConnection();
  int DoFullCheck();

  bool CheckBoarders();
  bool ListOfOccupiedFieldsDoVirtualMove(int RowFrom, int ColumnFrom, int RowTo, int ColumnTo);
  int  CheckForDoublets();

  void LoadListOfOccupiedFields();

  QList<int> ListOfTokenIDs;
  QList<int> ListOfCrownTokenIDs;


  QList<FieldWithToken> ListOfOccupiedFields;
};

#endif // RULECHECKER_H
