/////////////////////////////////////////////
//Klasse: Token
//Autor: Philipp Meiner
//Datum: 11.Dez.2012
/////////////////////////////////////////////
//Beschreibung:
//Die Objekte dieser Klasse knnen als Spielsteine oder Spielfeldmarkierungen benutzt werden.
//Sie knnen skaliert und Positioniert werden.
//
//Benutzung:
//Die Verwaltung der Tokens findet im Tokenmanager statt. Zum Positionieren, oder festlegen des Typs
//sollte der Tokenmanager verwendet werden.
//Die einzige Setter-Funktion die man nicht ber den Tokenmanager aufrufen kann und muss ist setVisibility.
//Alle Getter-Funktionen knnen jedoch jederzeit benutzt werden um Informationen ber die Token zu erhalten.


#ifndef TOKEN_H
#define TOKEN_H

#include <QObject>
#include <QGraphicsPixmapItem>
#include <QGraphicsScene>
#include <QPoint>
#include <QRect>
#include <QImage>

struct TokenType
{
  QImage* ImagePtr;
  int Player;
  int Layer;
  int ID;
  qreal ScaleFactor;
};


class Token : public QObject, private QGraphicsPixmapItem
{
  Q_OBJECT
public:
  Token(TokenType* TypePtr, QGraphicsScene* scene);
  ~Token();
  void CenterAtPos(QPoint Pos);
  void PlaceScaledInRectClean(QRect Rect);
  void PlaceScaledInRectFast(QRect Rect);
  void LoadImageScaled(qreal width, qreal height);

  int Player() const;

  int Type() const;
  void setType(TokenType * Type);

  int Row() const;
  int Column() const;
  void setColumn(int Column);
  void setRow(int Row);

  qreal MovePercent() const;
  void setMovePercent(qreal MovePercent);

  int MoveTargetRow() const;
  int MoveTargetColumn() const;
  void setMoveTarget(int MoveTargetRow, int MoveTargetColumn);
  QPoint Center() const;

  void setVisibility(bool IsVisible);
  bool Visibility() const;

  void setLayer(int Layer);
signals:

public slots:

private:
  TokenType *m_TokenTypePtr;

  int m_Row;
  int m_Column;

  qreal m_MovePercent;
  int m_MoveTargetRow;
  int m_MoveTargetColumn;
};


inline int Token::Row() const {return m_Row;}
inline void Token::setRow(int Row) {m_Row = Row;}

inline int Token::Column() const {return m_Column;}
inline void Token::setColumn(int Column) {m_Column = Column;}


inline qreal Token::MovePercent() const {return m_MovePercent;}
inline void Token::setMovePercent(qreal MovePercent) {m_MovePercent = MovePercent;}

inline int Token::MoveTargetRow() const {return m_MoveTargetRow;}
inline int Token::MoveTargetColumn() const {return m_MoveTargetColumn;}

inline void Token::setMoveTarget(int MoveTargetRow, int MoveTargetColumn) {
  m_MoveTargetRow = MoveTargetRow;
  m_MoveTargetColumn = MoveTargetColumn;
}

inline QPoint Token::Center() const {return QPoint(this->x()+this->pixmap().width()/2,this->y()+this->pixmap().height()/2);}

inline void Token::setVisibility(bool IsVisible){setVisible(IsVisible);}
inline bool Token::Visibility() const {return isVisible();}

inline void Token::setLayer(int Layer){setZValue((qreal)Layer);}
#endif // TOKEN_H
