/////////////////////////////////////////////
//Klasse: TokenManager
//Autor: Philipp Meiner
//Datum: 11.Dez.2012
////////////////////////////////////////////
//Beschreibung und Benutzungshinweise sind in der Header Datei.


#include "tokenmanager.h"
#include <qmath.h>
#include <QDebug>
#include <QGraphicsView>

TokenManager::TokenManager(Board *BoardPtr):TheBoard(BoardPtr)
{
  m_MovingInProgress = false;
  MovingSpeed = 10;
  connect(TheBoard,SIGNAL(CleanRedrawDone()),this,SLOT(CleanRedrawDone()));
  connect(&TokenMoveTimer,SIGNAL(timeout()),this,SLOT(MoveTimestep()));
}


int TokenManager::AddTokenType(const QString & fileName, int Player, int Layer, qreal ScaleFactor)
{
  ListOfTokenTypes.append(new TokenType());
  ListOfTokenTypes.last()->ImagePtr = new QImage(fileName);
  ListOfTokenTypes.last()->Player = Player;
  ListOfTokenTypes.last()->Layer = Layer;
  ListOfTokenTypes.last()->ID = ListOfTokenTypes.size()-1;
  ListOfTokenTypes.last()->ScaleFactor = ScaleFactor;
  return ListOfTokenTypes.last()->ID;
}


Token* TokenManager::getToken(int Nr)
{
  if((Nr >= 0)&&(Nr<ListOfTokens.size())){
    return ListOfTokens[Nr];
  }
  return 0;
}


Token* TokenManager::getTokenOnField(int Row, int Column, int Nr)
{
  int Counter = 0;
  for(int i = 0; i < ListOfTokens.size();i++){
    if((ListOfTokens[i]->Row() == Row)&&(ListOfTokens[i]->Column() == Column)){//Token befindet sich auf bergebenem Feld
      if(Counter == Nr){
        return ListOfTokens[i];
      }else{
        Counter++;
      }
    }
  }
  return 0;
}


Token* TokenManager::getTokenOfPlayer(int Player, int Nr)
{
  int Counter = 0;
  for(int i = 0; i < ListOfTokens.size();i++){
    if(ListOfTokens[i]->Player() == Player){
      if(Counter == Nr){
        return ListOfTokens[i];
      }else{
        Counter++;
      }
    }
  }
  return 0;
}


Token* TokenManager::getTokenWithType(int Type, int Nr)
{
  int Counter = 0;
  for(int i = 0; i < ListOfTokens.size();i++){
    if(ListOfTokens[i]->Type() == Type){
      if(Counter == Nr){
        return ListOfTokens[i];
      }else{
        Counter++;
      }
    }
  }
  return 0;
}


int TokenManager::getNrOfToken(Token* TokenPtr)
{
 return ListOfTokens.indexOf(TokenPtr);
}


int TokenManager::TotalTokensCount()
{
  return ListOfTokens.size();
}


int TokenManager::TokensCountForPlayer(int Player)
{
  int Counter = 0;
  for(int i = 0; i < ListOfTokens.size();i++){
    if(ListOfTokens[i]->Player() == Player){
        Counter++;
    }
  }
  return Counter;
}


int TokenManager::TokensCountForType(int Type)
{
  int Counter = 0;
  for(int i = 0; i < ListOfTokens.size();i++){
    if(ListOfTokens[i]->Type() == Type){
        Counter++;
    }
  }
  return Counter;
}


int TokenManager::TokensCountOnField(int Row, int Column)
{
  int Counter = 0;
  for(int i = 0; i < ListOfTokens.size(); i++){
    if((ListOfTokens[i]->Row() == Row)&&(ListOfTokens[i]->Column() == Column)){
        Counter++;
    }
  }
  return Counter;
}


void TokenManager::PlaceTokenOnField(int Nr, int Row, int Column)
{
   if((Row < 0)||(Row > TheBoard->RowCount()-1)||(Column < 0)||(Column > TheBoard->ColumnCount()-1)){
      qDebug()<<"ERROR - non existing field";
      return;
   }
   if((Nr >= 0)&&(Nr<ListOfTokens.size())){
      ListOfTokens[Nr]->PlaceScaledInRectClean(TheBoard->getFieldRect(Row,Column));
      ListOfTokens[Nr]->setRow(Row);
      ListOfTokens[Nr]->setColumn(Column);
      setTokenMoveDestination(Nr, Row, Column);
  }
}


void TokenManager::setTokenMoveDestination(int Nr, int Row, int Column)
{
   if((Row < 0)||(Row > TheBoard->RowCount()-1)||(Column < 0)||(Column > TheBoard->ColumnCount()-1)){
      qDebug()<<"ERROR - non existing field";
      return;
   }
   if((Nr >= 0)&&(Nr<ListOfTokens.size())){
      if((m_MovingInProgress)&&(ListOfTokens[Nr]->MovePercent() < 100)){
        qDebug()<<"ERROR - setting of destination not allowed during move";
        return;
      }
      ListOfTokens[Nr]->setMoveTarget(Row, Column);

      if((ListOfTokens[Nr]->Row() != ListOfTokens[Nr]->MoveTargetRow())||(ListOfTokens[Nr]->Column() != ListOfTokens[Nr]->MoveTargetColumn())){//Zielfeld ist nicht aktuelles Feld
        ListOfTokens[Nr]->setMovePercent(0);
      }else{
        ListOfTokens[Nr]->setMovePercent(101); //Bewegung nicht ntig
      }
  }
}


Token* TokenManager::AddToken(int Type, int Row, int Column)
{
  if((Type < 0)||(Type > ListOfTokenTypes.size()-1)){return 0;}

  ListOfTokens.append(new Token(ListOfTokenTypes[Type],TheBoard->viewPtr()->scene()));
  PlaceTokenOnField(ListOfTokens.size()-1,Row,Column);
  return ListOfTokens.last();
}


 void TokenManager::DeleteTokenWithNr(int Nr){
  if((Nr >= 0)&&(Nr<ListOfTokens.size())){//Nummer vorhanden
    ListOfTokens[Nr]->deleteLater(); //deleteLater des Objekts aufrufen. Um die korrekte Lschung kmmet sich dann das Qt-Framework.
    ListOfTokens.removeAt(Nr);
  }
}


void TokenManager::DeleteToken(Token* TokenPtr)
{
  int TokenNr = getNrOfToken(TokenPtr);
  if(TokenNr >= 0){
    DeleteTokenWithNr(TokenNr);
  }
}


void TokenManager::setTokenType(int TokenNr, int TypeNr)
{
  if((TokenNr >= 0)&&(TokenNr<ListOfTokens.size())&&(TypeNr >= 0)&&(TypeNr < ListOfTokenTypes.size())){
    QPoint TempPoint = ListOfTokens[TokenNr]->Center();
    ListOfTokens[TokenNr]->setType(ListOfTokenTypes[TypeNr]);
    ListOfTokens[TokenNr]->LoadImageScaled(TheBoard->FieldWidth(),TheBoard->FieldHeight());
    ListOfTokens[TokenNr]->CenterAtPos(TempPoint);
  }
}


void TokenManager::ReplaceAllTokensClean()
{
  for(int i = 0; i < ListOfTokens.size(); i++){
      ListOfTokens[i]->PlaceScaledInRectClean(TheBoard->getFieldRect(ListOfTokens[i]->Row(),ListOfTokens[i]->Column()));
  }
}


void TokenManager::ReplaceAllTokensFast()
{
    for(int i = 0; i < ListOfTokens.size();i++){
      ListOfTokens[i]->PlaceScaledInRectFast(TheBoard->getFieldRect(ListOfTokens[i]->Row(),ListOfTokens[i]->Column()));
    }
}


void TokenManager::CleanRedrawDone()
{
  ReplaceAllTokensClean();
}


//Diese Funktion wird whrend der Bewegung zyklisch ausgefhrt
void TokenManager::AdvanceToken(Token* TokenToMove)
{
  qreal NewX,NewY,Dist;
  QRect StartRect, DestRect;
  TokenToMove->setLayer(ListOfTokenTypes[TokenToMove->Type()]->Layer+1);//Layer erhhen, damit bewegte Token sich BER stehende bewegen.

  //Die Bewegung wird durch Interpolation zwischen Start und Zielpunkt erreicht. Damit sie stets gleich schnell ist, muss das Fortschreiten
  //der Interpoaltion abhngig von der Entfernung stattfinden.
  Dist = qSqrt(qPow(TokenToMove->MoveTargetRow() - TokenToMove->Row(),2)+qPow(TokenToMove->MoveTargetColumn() - TokenToMove->Column(),2));
  TokenToMove->setMovePercent(TokenToMove->MovePercent() + MovingSpeed/Dist);

  StartRect = TheBoard->getFieldRect(TokenToMove->Row(), TokenToMove->Column());
  DestRect  = TheBoard->getFieldRect(TokenToMove->MoveTargetRow(), TokenToMove->MoveTargetColumn());

  if(TokenToMove->MovePercent() < 100){//Bewegung noch nicht abgeschlossen
    NewX = StartRect.center().x() + 0.01*TokenToMove->MovePercent()*(DestRect.center().x()-StartRect.center().x());
    NewY = StartRect.center().y() + 0.01*TokenToMove->MovePercent()*(DestRect.center().y()-StartRect.center().y());
    TokenToMove->CenterAtPos(QPoint(NewX, NewY));
  }else{
    TokenToMove->setRow(TokenToMove->MoveTargetRow());
    TokenToMove->setColumn(TokenToMove->MoveTargetColumn());
    TokenToMove->PlaceScaledInRectClean(DestRect);
    TokenToMove->setLayer(ListOfTokenTypes[TokenToMove->Type()]->Layer);
  }
}


//Diese Methode triggert das Zyklische Aufrufen der Methode AdvanceToken whrend der Bewegung
void TokenManager::MoveTimestep()
{
  bool MovingFinished = true;
  for(int i = 0; i < ListOfTokens.size(); i++){
    if((ListOfTokens[i]->MovePercent() < 100)){
      AdvanceToken(ListOfTokens[i]);
      MovingFinished = false;
    }
  }
  if(MovingFinished){//Jeder Stein hat sein Ziel erreicht
    TokenMoveTimer.stop();
    m_MovingInProgress = false;
    emit MovingDone();
  }
}


void TokenManager::MoveTokens()
{
  m_MovingInProgress = true;
  TokenMoveTimer.start(20);
}


void TokenManager::setMovingSpeed(int Speed){
  MovingSpeed = Speed;
}
