/////////////////////////////////////////////
//Klasse: Token
//Autor: Philipp Meiner
//Datum: 11.Dez.2012
/////////////////////////////////////////////
//Beschreibung:
//Diese Klasse Verwaltet die Token-Objekte in einer Liste. Sie bietet Funktionen um Token zu finden und zu Selektieren.
//Auch die Erstellung neuer Tokens oder das Lschen vorhandener wird mit dieser Klasse durchgefhrt.
//Ebenso das Positionieren der Token. Die Klasse kmmert sich auch um das Skalieren und Neupositionieren
//nachdem die Fenstergre gendert wurde
//
//Benutzung:
//Bei der Erstellung wird der Klasse ein Zeiger auf das Board-Objekt bergeben, auf dem die Tokens dargestellt werden.
//Bevor Tokens erstellt werden knnen, mssen zunchst TokenTypes erstellt werden. Dafr wird die Methode AddTokenType verwendet.
//Dieser muss ein String mit dem Speicherort und dem Namen der Grafikdatei bergeben werden. Die Angaben einer Spielernummer, eines
//Layers und eines Skalierungsfaktors sind Optional. Der Skalierungsfaktor gibt an, wie gro die Grafik im Vergleich zu der Gre
//eines Spielfeldes skaliert wird. Bei einem Wert von 1 (default) fllt die Grafik ein Spielfeld genau aus.
//Die Methode AddTokenType gibt die Indexnummer des von ihr erstellten TokenTypes zurck.
//
//Ist mindestens ein TokenType definiert, kann mit der Methode AddToken ein neuer Token erstellt werden. Der Methode muss die Index-
//Nummer eines existierenden TokenTypes bergeben werden. Optional kann auch ein Spielfeld angegeben werden, auf dem der Token platziert wird.
//
//Zur Verwaltung der Token gibt verschiedene Methoden. Die 4 Methoden mit "Count" im Namen geben die Anzahl gerade existierenden Token
//Mit der entsprechenden Eigenschaft an. Mit den "getToken"-Metoden kann man einen Zeiger auf einen Token mit der entsprechenden Eigenschaft
//bekommen. Gibt es dabei mehr als einen, kann man den entsprechenden Token ber eine bergebene Nummer auswhlen. Um fr alle Token mit einer
//bestimmten Eigenschaft eine Operation durchzufhren, muss man also erst deren Anzahl bestimmen und sie dann in einer Schleife einzeln Anfordern.
//
//Man kann auch von einem Zeiger auf einen Token auf seine Nummer in der Liste aller Tokens schlieen und Umgekehrt. Dafr sind die Methoden
//getNrOfToken bzw. getToken. Wobei hier die getToken-Methode ohne zustzliche Angabe einer Eigenschaft gemeint ist.
//
//Um einen Token animiert zu verschieben, muss man zuerst mit der Methode setTokenMoveDestination das Ziel der Bewegung festlegen. Anschlieend
//kann mit der Methode MoveTokens die Bewegungsanimation fr alle Token mit einem Ziel gestartet werden.
//
//Um alle Token neu zu skalieren un zu Positioniern (z.B. nach einem ndern der Fenstergre) kann man die ReplaceAllTokens-Methoden aufrufen.
//Die Fast-Variante ist dabei sehr schnell aber unsauber. Sie soll verwendet werden, wenn sich die Gre stndig ndert. Zum Beispiel wenn die
//Gre des Fensters durch ziehen mit der Maus gendert wird. Ist die Grennderung abgeschlossen, sollten alle Tokens mit der Clean-Variante
//neu positioniert werden.


#ifndef TOKENMANAGER_H
#define TOKENMANAGER_H

#include <QList>
#include <QImage>
#include <QTimer>
#include "token.h"
#include "board.h"

class TokenManager : public QObject
{
  Q_OBJECT
public:
  TokenManager(Board* BoardPtr);
  int AddTokenType(const QString & fileName, int Player = 0, int Layer = 0, qreal ScaleFactor = 1);

  Token* AddToken(int Type, int Row = 0, int Column = 0);

  Token* getToken(int Nr);
  Token* getTokenOnField(int Row, int Column, int Nr = 0);
  Token* getTokenOfPlayer(int Player, int Nr = 0);
  Token* getTokenWithType(int Type, int Nr = 0);

  int getNrOfToken(Token* TokenPtr);

  void setTokenType(int TokenNr, int TypeNr);
  void PlaceTokenOnField(int Nr, int Row, int Column);
  void setTokenMoveDestination(int Nr, int Row, int Column);

  int TotalTokensCount();
  int TokensCountForPlayer(int Player);
  int TokensCountForType(int Type);
  int TokensCountOnField(int Row, int Column);

  void DeleteTokenWithNr(int Nr);
  void DeleteToken(Token* TokenPtr);

  void ReplaceAllTokensClean();
  void ReplaceAllTokensFast();

  void MoveTokens();
  void setMovingSpeed(int Speed);
  bool MovingInProgress() const;

  int RowCount() const;
  int ColumnCount() const;

private:
  bool m_MovingInProgress;
  int MovingSpeed;
  QTimer TokenMoveTimer;

  QList<Token*> ListOfTokens;
  QList<TokenType*> ListOfTokenTypes;
  Board *TheBoard;

  void AdvanceToken(Token* TokenToMove);

signals:
  void MovingDone();

private slots:
  void CleanRedrawDone();
  void MoveTimestep();
};

inline bool TokenManager::MovingInProgress() const {return m_MovingInProgress;}

inline int TokenManager::RowCount() const {return TheBoard->RowCount();}
inline int TokenManager::ColumnCount() const {return TheBoard->ColumnCount();}

#endif // TOKENMANAGER_H
